#include "ADC.h"
#include "IO.h"
#include "PWM.h"
#include <avr\io.h>
#include <avr/interrupt.h>

/******************************************************************/
/* ADC done INT                                                   */ 
/******************************************************************/
ISR(ADC_vect )
{
 // get values
 unsigned char t8;
 unsigned int  t16;
 
 t8=ADCL;
 t16=ADCH;
 t16=t16 << 8;
 t16+=t8;
 PWM_ADCResult_POT=t16;
 
 PWM_Flagreg|=PWM_Flag_ADCPOT;     // SET flag
 
}
/******************************************************************/
/*                                                                */ 
/******************************************************************/
void ADC_INIT(void)
{
  
  ADCSRA |=(1<<ADPS2)+(1<<ADPS1)+(1<<ADPS0);    // MCLK/128 = 156 kHz -> fs=12kHz==83s
  //ADCSRA |=(1<<ADPS2)+(1<<ADPS1);             // MCLK/64  = 312 kHz -> fs=24kHz==42s
  
  ADCSRA |=(1<<ADIE);                           // INT enable
  ADCSRB &=~(1<<ADLAR);                         // RJ result   

  ADCSRA |=(1<<ADEN);
  DIDR0  |=(1<<ADC0D);

  
  ADC_MUX_CLR;
  ADC_MUX_POT;

  #ifdef MODE_RampupDwn_ON
  Ramp_Mode=LPWM_Ramp_Mode_OFF;
  #endif

  LPWM_Drivestep=0;
  PWM_Flagreg&=~PWM_Flag_ADCPOT;               // CLR Flag
  ADCSRA |=(1<<ADSC);                          // Start ADC
}



/******************************************************************/
/*                                                                */ 
/******************************************************************/
void ADC_POT_Task(void)
{
  unsigned char Stepnew;
  unsigned char t8;
  
  
  /* Ramp  up/down */
  #ifdef MODE_RampupDwn_ON
  if ((Ramp_Mode==LPWM_Ramp_Mode_UP)&&(!Timer_Rampup))
  {
        ADC_ReSETRamptimer();
        t8=Rampup_StepTemp;
        if (Rampup_StepTemp<Step)
        {
            t8++;
            ADC_Drivestep2Settings(t8,&Stepnew);      
            PWM_LPWM_SetPWM(Stepnew);                      // set new value (LPWM)
            Rampup_StepTemp=t8;   
        }
  }
  if ((Ramp_Mode==LPWM_Ramp_Mode_DWN)&&(!Timer_Rampdwn))
  {
        ADC_ReSETRamptimer();
        t8=Rampdwn_StepTemp;
        if (t8>Step)
        {
            t8--;
            ADC_Drivestep2Settings(t8,&Stepnew);      
            PWM_LPWM_SetPWM(Stepnew);                      // set new value (LPWM)
            Rampdwn_StepTemp=t8;
        }

  }
  #endif
  
  
  /* POT Value */
  if (!(PWM_Flagreg&PWM_Flag_ADCPOT))      return;  // FLAG NOT set ?? -> exit
  if (PWM_Flagreg& PWM_Flag_CTRLI2C)       return;  // ControlMode = I2C ? 

  PWM_Flagreg&=~PWM_Flag_ADCPOT;               // CLR Flag
  
  ADCSRA |=(1<<ADSC);                          // Start ADC
  ADC_Potcnt++;                                // inc counter
  
  #ifdef DEBUG_POT
  ADC_Potbuf[ADC_Potcnt]=PWM_ADCResult_POT;
  #endif
  
  if (ADC_Potcnt>7)                            // accu done ?
  {
    ADC_Potvalmean=0;
    for (ADC_Potcnt=1;ADC_Potcnt<9;ADC_Potcnt++)
    {
      ADC_Potvalmean+=ADC_Potbuf[ADC_Potcnt];
    }
    
    ADC_Potcnt=0;                             // clr counter
    ADC_Potvalmean=ADC_Potvalmean >> 3;       // div by 8

    if (ADC_Potvalmean>10)
    {
      if ((ADC_Potvalmean-10>ADC_Potvalmean_OLD) || (ADC_Potvalmean+10<ADC_Potvalmean_OLD))
      {
        ADC_Potvalmean_OLD=ADC_Potvalmean;      // more than 10% movement
      }
      else
      {
       ADC_Potvalmean=ADC_Potvalmean_OLD;      // less than 10% movement
      }  
    }
    else
    {
      ADC_Potvalmean=0;
    }
      
     Stepnew=ADC_Potvale2Drivestep(ADC_Potvalmean); // ADCvalue 2 step
     //Stepnew=ADC_Potvale2Drivestep(tPotmean); // ADCvalue 2 step
     
     #ifndef DEBUG_VPWM 
     ADC_Potvalmean=0;
     #endif

     
     if (Stepnew==LPWM_Drivestep) return;
     
     #ifdef MODE_RampupDwn_ON
     if(Step>Stepnew) Ramp_Mode=LPWM_Ramp_Mode_DWN;
     if(Step<Stepnew) Ramp_Mode=LPWM_Ramp_Mode_UP;
     #endif
     LPWM_Drivestep=Stepnew;  

     #ifndef DEBUG_VPWM
     //ADC_Drivestep2Settings(Step,&t8);              // step 2 parameters
     
     
     
     #ifdef MODE_RampupDwn_ON
     if (Ramp_Mode==LPWM_Ramp_Mode_OFF)
     {
     #endif


     PWM_SetNew(LPWM_Drivestep);

     #ifdef MODE_RampupDwn_ON  
     }
     #endif
     #endif
     
     #ifdef DEBUG_VPWM
     ADC_Potvalmean=ADC_Potvalmean >> 3;        // div by 8
     if (ADC_Potvalmean >VPWM_PWM_Width-3) ADC_Potvalmean=VPWM_PWM_Width-3 ;
     if (!ADC_Potvalmean)  ADC_Potvalmean=8;
     OCR0A=0xFF-ADC_Potvalmean;
     ADC_Potvalmean=0;
     #endif

  }
  


}

//*******************************************************************//
//*** Poti to Step
//*******************************************************************//
unsigned char ADC_Potvale2Drivestep(unsigned int Potvalue)
{
  unsigned char Step;
  if (Potvalue>=ADC_Potval_Step27_max)  Step=28;
  if (Potvalue<ADC_Potval_Step27_max)  Step=27;
  if (Potvalue<ADC_Potval_Step26_max)  Step=26;
  if (Potvalue<ADC_Potval_Step25_max)  Step=25;
  if (Potvalue<ADC_Potval_Step24_max)  Step=24;
  if (Potvalue<ADC_Potval_Step23_max)  Step=23;
  if (Potvalue<ADC_Potval_Step22_max)  Step=22;
  if (Potvalue<ADC_Potval_Step21_max)  Step=21;
  if (Potvalue<ADC_Potval_Step20_max)  Step=20;
  if (Potvalue<ADC_Potval_Step19_max)  Step=19;
  if (Potvalue<ADC_Potval_Step18_max)  Step=18;
  if (Potvalue<ADC_Potval_Step17_max)  Step=17;
  if (Potvalue<ADC_Potval_Step16_max)  Step=16;
  if (Potvalue<ADC_Potval_Step15_max)  Step=15;
  if (Potvalue<ADC_Potval_Step14_max)  Step=14;
  if (Potvalue<ADC_Potval_Step13_max)  Step=13;
  if (Potvalue<ADC_Potval_Step12_max)  Step=12;
  if (Potvalue<ADC_Potval_Step11_max)  Step=11;
  if (Potvalue<ADC_Potval_Step10_max)  Step=10;
  if (Potvalue<ADC_Potval_Step9_max)  Step=9;
  if (Potvalue<ADC_Potval_Step8_max)  Step=8;
  if (Potvalue<ADC_Potval_Step7_max)  Step=7;
  if (Potvalue<ADC_Potval_Step6_max)  Step=6;
  if (Potvalue<ADC_Potval_Step5_max)  Step=5;
  if (Potvalue<ADC_Potval_Step4_max)  Step=4;
  if (Potvalue<ADC_Potval_Step3_max)  Step=3;
  if (Potvalue<ADC_Potval_Step2_max)  Step=2;
  if (Potvalue<ADC_Potval_Step1_max)  Step=1;
  if (Potvalue<ADC_Potval_Step0_max)  Step=0;

  return(Step);

}
//*******************************************************************//
//*** Step to PWM
//*******************************************************************//
void ADC_Drivestep2Settings(unsigned char Step,unsigned char *PWM_DC)
{
  /***********************************************************/
  #ifdef MODE_Rangiergang
  if (PWM_Flagreg & PWM_Flag_Rangier)
  {
  switch(Step)
  {
    default:         *PWM_DC=0;break;
    case 1:          *PWM_DC=PWM_R_STEP1;break;
    case 2:          *PWM_DC=PWM_R_STEP2;break;
    case 3:          *PWM_DC=PWM_R_STEP3;break;
    case 4:          *PWM_DC=PWM_R_STEP4;break;
    case 5:          *PWM_DC=PWM_R_STEP5;break;
    case 6:          *PWM_DC=PWM_R_STEP6;break;
    case 7:          *PWM_DC=PWM_R_STEP7;break;
    case 8:          *PWM_DC=PWM_R_STEP8;break;
    case 9:          *PWM_DC=PWM_R_STEP9;break;
    case 10:         *PWM_DC=PWM_R_STEP10;break;
    case 11:         *PWM_DC=PWM_R_STEP11;break;
    case 12:         *PWM_DC=PWM_R_STEP12;break;
    case 13:         *PWM_DC=PWM_R_STEP13;break;
    case 14:         *PWM_DC=PWM_R_STEP14;break;
    case 15:         *PWM_DC=PWM_R_STEP15;break;
    case 16:         *PWM_DC=PWM_R_STEP16;break;
    case 17:         *PWM_DC=PWM_R_STEP17;break;
    case 18:         *PWM_DC=PWM_R_STEP18;break;
    case 19:         *PWM_DC=PWM_R_STEP19;break;
    case 20:         *PWM_DC=PWM_R_STEP20;break;
    case 21:         *PWM_DC=PWM_R_STEP21;break;
    case 22:         *PWM_DC=PWM_R_STEP22;break;
    case 23:         *PWM_DC=PWM_R_STEP23;break;
    case 24:         *PWM_DC=PWM_R_STEP24;break;
    case 25:         *PWM_DC=PWM_R_STEP25;break;
    case 26:         *PWM_DC=PWM_R_STEP26;break;
    case 27:         *PWM_DC=PWM_R_STEP27;break;
    case 28:         *PWM_DC=PWM_R_STEP28;break;
  } 
  
    return;
  }
  #endif
  /***********************************************************/
  switch(Step)
  {
    default:         *PWM_DC=0;break;
    case 1:          *PWM_DC=PWM_STEP1;break;
    case 2:          *PWM_DC=PWM_STEP2;break;
    case 3:          *PWM_DC=PWM_STEP3;break;
    case 4:          *PWM_DC=PWM_STEP4;break;
    case 5:          *PWM_DC=PWM_STEP5;break;
    case 6:          *PWM_DC=PWM_STEP6;break;
    case 7:          *PWM_DC=PWM_STEP7;break;
    case 8:          *PWM_DC=PWM_STEP8;break;
    case 9:          *PWM_DC=PWM_STEP9;break;
    case 10:         *PWM_DC=PWM_STEP10;break;
    case 11:         *PWM_DC=PWM_STEP11;break;
    case 12:         *PWM_DC=PWM_STEP12;break;
    case 13:         *PWM_DC=PWM_STEP13;break;
    case 14:         *PWM_DC=PWM_STEP14;break;
    case 15:         *PWM_DC=PWM_STEP15;break;
    case 16:         *PWM_DC=PWM_STEP16;break;
    case 17:         *PWM_DC=PWM_STEP17;break;
    case 18:         *PWM_DC=PWM_STEP18;break;
    case 19:         *PWM_DC=PWM_STEP19;break;
    case 20:         *PWM_DC=PWM_STEP20;break;
    case 21:         *PWM_DC=PWM_STEP21;break;
    case 22:         *PWM_DC=PWM_STEP22;break;
    case 23:         *PWM_DC=PWM_STEP23;break;
    case 24:         *PWM_DC=PWM_STEP24;break;
    case 25:         *PWM_DC=PWM_STEP25;break;
    case 26:         *PWM_DC=PWM_STEP26;break;
    case 27:         *PWM_DC=PWM_STEP27;break;
    case 28:         *PWM_DC=PWM_STEP28;break;
  }
}


//*******************************************************************//
//*** Setting Ramptimer
//*******************************************************************//
void ADC_ReSETRamptimer(void)
{
    #ifdef MODE_RampupDwn_ON
    unsigned int t16;
    
    switch(Rampup_Mode)
    {
        default:    break;
        case LPWM_Rampmode_250m: Timer_Rampup=LPWM_Ramptime_250m;break; 
        case LPWM_Rampmode_500m: Timer_Rampup=LPWM_Ramptime_500m;break;
        case LPWM_Rampmode_1s:   Timer_Rampup=LPWM_Ramptime_1s;  break;
         
    
    }
    switch(Rampdwn_Mode)
    {
        default:    break;
        case LPWM_Rampmode_250m: Timer_Rampdwn=LPWM_Ramptime_250m;break;
        case LPWM_Rampmode_500m: Timer_Rampdwn=LPWM_Ramptime_500m;break;
        case LPWM_Rampmode_1s:   Timer_Rampdwn=LPWM_Ramptime_1s;  break;
    
    }
    #endif

}

