

#include "I2C.h"
#include "IO.h"
#include "ADC.h"
#include "PWM.h"
#include <avr\io.h>
#include <avr/interrupt.h>

static volatile unsigned char USI_TWI_Overflow_State;

/*********************************************************************************/
/***                                                                           ***/
/*********************************************************************************/

void I2C_INIT(void)
{
  
  unsigned char ADR;
  Flush_TWI_Buffers();
  
  /*TWI_slaveAddress=EEP_Readbyte(EEP_Adress_I2CADR);
  if (!TWI_slaveAddress)        TWI_slaveAddress=TWI_MYAddress;
  if (TWI_slaveAddress>0x7F)    TWI_slaveAddress=0x7F;
  */
  ADR=EEP_Readbyte(EEP_Adress_I2CADR);
  //if (!ADR)        ADR=TWI_MYAddress;
  if (!ADR) ADR=0x61;
  if (ADR>0x7F)    ADR=0x61;
  TWI_slaveAddress=ADR;
  

  PORT_USI |=  (1<<PORT_USI_SCL);                                 // Set SCL high
  PORT_USI |=  (1<<PORT_USI_SDA);                                 // Set SDA high
  DDR_USI  |=  (1<<PORT_USI_SCL);                                 // Set SCL as output
  DDR_USI  &= ~(1<<PORT_USI_SDA);                                 // Set SDA as input
  USICR    =  (1<<USISIE)|(0<<USIOIE)|                            // Enable Start Condition Interrupt. Disable Overflow Interrupt.
              (1<<USIWM1)|(0<<USIWM0)|                            // Set USI in Two-wire mode. No USI Counter overflow prior
                                                                  // to first Start Condition (potentail failure)
              (1<<USICS1)|(0<<USICS0)|(0<<USICLK)|                // Shift Register Clock Source = External, positive edge
              (0<<USITC);
  USISR    = 0xF0;                                                // Clear all flags and reset overflow counter
  USI_TWI_Overflow_State=USI_SLAVE_StateNone; 
}

/*********************************************************************************/
/***                                                                           ***/
/*********************************************************************************/

void I2C_TASK(void)
{
   static volatile unsigned char Dcnt;
   
   unsigned char t_Flag;
   unsigned char t_CMD;
   unsigned char t_CHKSUM;
   unsigned char t_Data;
   unsigned char t_temp;
   unsigned char SUM;
   unsigned char cnt; 
    
   if (USI_TWI_GetnumofDataInReceiveBuffer() < TWI_PackLen) return;           // exit if no new packet
   
   SUM=TWI_slaveAddress;
   for (cnt=0;cnt<(TWI_PackLen-1);cnt++)     SUM+=TWI_RxBuf[cnt];
   
   Dcnt++;
   
   t_Flag=TWI_RxBuf[TWI_PPos_FLAG];
  
   TWI_RxBuf[TWI_PPos_FLAG]&=~TWI_FLAG_SLAVEREADY;      // clr ready Flag
   t_CMD=TWI_RxBuf[TWI_PPos_TYPE];
   t_Data=TWI_RxBuf[TWI_PPos_DATA];
   t_CHKSUM=TWI_RxBuf[TWI_PPos_CHKSUM]; 
   Flush_TWI_RxBuffer();  // clr buffer
 
   
   //if (TWI_CalcChecksum(&TWI_RxBuf,TWI_MYAddress,TWI_PackLen-1) == t_CHKSUM)
   
   if (SUM == t_CHKSUM)
    {         
        // CHECKSUM OK
        TWI_RxBuf[TWI_PPos_FLAG]|=TWI_FLAG_CHECKOK;    // set flag
        TWI_RegisterPointer=t_CMD;
        if (TWI_RxBuf[TWI_PPos_FLAG] & TWI_FLAG_DATAVALID)
        {
            
            TWI_RxBuf[TWI_PPos_DATA]=Dcnt; 
            // DATA is valid 
            switch(t_CMD)
            {
                default:            break;
                case TWI_CMD_DIR:   
                                    if (t_Data)
                                    {
                                      PWM_Flagreg|=PWM_Flag_CTRL_DIR;
                                    }
                                    else
                                    {
                                      PWM_Flagreg&=~PWM_Flag_CTRL_DIR;
                                    }

                                    break;
                case TWI_CMD_SPEED: //Step=t_Data;
                                    PWM_SetNew(t_Data);
                                     break;
                case TWI_CMD_FREQ:  
                                    LPWM_FMode=t_Data;
                                    if (PWM_Flagreg & PWM_Flag_CTRLI2C)
                                    {
                                      PWM_SetNew(0);
                                    }
                                    else
                                    {
                                      PWM_SetNew(LPWM_Drivestep);
                                    }
                                    break;
                case TWI_CMD_MODE:
                                    LPWM_DMode=t_Data;
                                    PWM_SetNew(LPWM_Drivestep);
                                    break;
                case TWI_CMD_VOLT:  LPWM_Voltage=t_Data;
                                    LPWM_VOCR=PWM_VStep2VPWM(LPWM_Voltage);
                                    PWM_SetNew(LPWM_Drivestep);
                                    LPWM_VOCR=PWM_VStep2VPWM(LPWM_Voltage);
                                    OCR0A=LPWM_VOCR;
                                    break;  
                #ifdef MODE_Rangiergang
                case TWI_CMD_Rangier:
                                    if (t_Data)
                                    {
                                      PWM_Flagreg|=PWM_Flag_Rangier;
                                    }
                                    else
                                    {
                                      PWM_Flagreg&=~PWM_Flag_Rangier;
                                    }
                                    break;
                #endif                                                                       
                case TWI_CMD_RAMPUP:
                                    break;  
                case TWI_CMD_RAMPDWN:
                                    break;
                case TWI_CMD_CONTROL:
                                    if (t_Data)
                                    {
                                      PWM_Flagreg|=PWM_Flag_CTRLI2C;
                                    }
                                    else
                                    {
                                      PWM_Flagreg&=~PWM_Flag_CTRLI2C;
                                    }
                                    break;                                      
                case TWI_CMD_NEWI2CA:
                                    if ((t_Data) && (t_Data < 0x80))
                                    {
                                        EEP_Writebyte(EEP_Adress_I2CADR,t_Data);
                                        TWI_slaveAddress=t_Data;
                                    }    
                                    break;  
            }        
        }
        else
        {
            // DATA invalid -> set pointer !
             switch(t_CMD)
            {
                default:            break;
                case TWI_CMD_DIR:   TWI_RxBuf[TWI_PPos_DATA]=PWM_Flagreg & PWM_Flag_CTRL_DIR;
                                    break;                  
                case TWI_CMD_SPEED: TWI_RxBuf[TWI_PPos_DATA]=LPWM_Drivestep;
                                    break;
                case TWI_CMD_FREQ:  TWI_RxBuf[TWI_PPos_DATA]=LPWM_FMode;
                                    break;
                case TWI_CMD_MODE:
                                    break;
                case TWI_CMD_VOLT:  TWI_RxBuf[TWI_PPos_DATA]=LPWM_Voltage;
                                    break;  
                case TWI_CMD_RAMPUP:
                                    break;  
                case TWI_CMD_RAMPDWN:
                                    break;
                case TWI_CMD_CONTROL:TWI_RxBuf[TWI_PPos_DATA]=PWM_Flagreg & PWM_Flag_CTRLI2C;
                                    break;                         
                case TWI_CMD_FASTRead:
                                    TWI_RxBuf[TWI_PPos_DATA]=LPWM_Drivestep;
                                    if (PWM_Flagreg & PWM_Flag_CTRL_DIR) TWI_RxBuf[TWI_PPos_DATA] |= (1<<5);
                                    if (PWM_Flagreg & PWM_Flag_CTRL_ON)  TWI_RxBuf[TWI_PPos_DATA] |= (1<<6);                                    
                                    break;                                                 
                case TWI_CMD_NEWI2CA:TWI_RxBuf[TWI_PPos_DATA]=0xFF;
                                    break;  
                               
            }        
        }
        
          SUM=TWI_slaveAddress;
          for (cnt=0;cnt<(TWI_PackLen-1);cnt++)     SUM+=TWI_RxBuf[cnt];
          SUM+=TWI_FLAG_SLAVEREADY;
          TWI_RxBuf[TWI_PPos_CHKSUM]=SUM;
   }
   else
   {
        // CHECKSUM WRONG
        TWI_RxBuf[TWI_PPos_FLAG]&=~TWI_FLAG_CHECKOK;   // clr flag  
   }
   TWI_RxBuf[TWI_PPos_FLAG]|=TWI_FLAG_SLAVEREADY;      // set ready Flag
   
}

/*********************************************************************************/
/***                                                                           ***/
/*********************************************************************************/
/*
unsigned char TWI_CalcChecksum(unsigned char *buffer,unsigned char ADR, unsigned char LEN)
{
    unsigned char SUM;
    unsigned char cnt;
    
    SUM=ADR;
    for (cnt=0;cnt<LEN;cnt++)
    {
        SUM+=*(buffer+cnt);
    }
    return(SUM);
}
*/
/*********************************************************************************/
/***                                                                           ***/
/*********************************************************************************/
void Flush_TWI_Buffers(void)
{
    TWI_Rxptr = 0;
    TWI_Txptr = 0;
}
void Flush_TWI_RxBuffer(void)
{
    TWI_Rxptr = 0;
}
/*********************************************************************************/
/***                                                                           ***/
/*********************************************************************************/
unsigned char USI_TWI_GetnumofDataInReceiveBuffer( void )
{
    return(TWI_Rxptr);                             
}

/*********************************************************************************/
/*** USI START cond.                                                           ***/
/*********************************************************************************/
ISR(SIG_USI_START)
{
    unsigned char tmpUSISR;                                         // Temporary variable to store volatile
    tmpUSISR = USISR;                                               // Not necessary, but prevents warnings
    // Set default starting conditions for new TWI package
    USI_TWI_Overflow_State = USI_SLAVE_CHECK_ADDRESS;
    DDR_USI  &= ~(1<<PORT_USI_SDA);                                 // Set SDA as input
#ifndef USI_DONBLAKE    
    while ( (PIN_USI & (1<<PORT_USI_SCL)) & !(tmpUSISR & (1<<USIPF)) );   // Wait for SCL to go low to ensure the "Start Condition" has completed.
    USICR   =   (1<<USISIE)|(1<<USIOIE)|                            // Enable Overflow and Start Condition Interrupt. (Keep StartCondInt to detect RESTART)
                (1<<USIWM1)|(1<<USIWM0)|                            // Set USI in Two-wire mode.
                (1<<USICS1)|(0<<USICS0)|(0<<USICLK)|                // Shift Register Clock Source = External, positive edge
                (0<<USITC);
    USISR  =    (1<<USI_START_COND_INT)|(1<<USIOIF)|(1<<USIPF)|(1<<USIDC)|      // Clear flags
                (0x0<<USICNT0);                                     // Set USI to sample 8 bits i.e. count 16 external pin toggles.
#endif
#ifdef USI_DONBLAKE
    while ( (PIN_USI & (1<<PORT_USI_SCL)) && (!(PIN_USI & (1<<PORT_USI_SDA))));
    if (!( PIN_USI & ( 1 << PIN_USI_SDA )))
    {
           // NO Stop
      USICR   = (1<<USISIE)|(1<<USIOIE)|                            // Enable Overflow and Start Condition Interrupt. (Keep StartCondInt to detect RESTART)
                (1<<USIWM1)|(1<<USIWM1)|                            // Set USI in Two-wire mode.
                (1<<USICS1)|(0<<USICS0)|(0<<USICLK)|                // Shift Register Clock Source = External, positive edge
                (0<<USITC);
    }
    else
    {     // STOP
      USICR   = (1<<USISIE)|(0<<USIOIE)|                            // Enable Overflow and Start Condition Interrupt. (Keep StartCondInt to detect RESTART)
                (1<<USIWM1)|(0<<USIWM1)|                            // Set USI in Two-wire mode.
                (1<<USICS1)|(0<<USICS0)|(0<<USICLK)|                // Shift Register Clock Source = External, positive edge
                (0<<USITC);
    
    }
    USISR  =    (1<<USI_START_COND_INT)|(1<<USIOIF)|(1<<USIPF)|(1<<USIDC)|      // Clear flags
                (0x0<<USICNT0);                                     // Set USI to sample 8 bits i.e. count 16 external pin toggles.    

#endif

}


/*! \brief USI counter overflow ISR
 * Handels all the comunication. Is disabled only when waiting
 * for new Start Condition. */
 
/*********************************************************************************/
/*** USI CNTer overflow -> New Data                                            ***/
/*********************************************************************************/
ISR(SIG_USI_OVERFLOW)
{
  unsigned int t16;

  #ifdef TWI_DelayINT_ON
  _TWIdelay_loop(TWI_DelayINT); // simple but time consuming
  #endif
  
  /*
  USISR|=(1<<USIOIF);           // clear "Overflow INT" Flag -> release SCL
  _TWIdelay_loop(TWI_DelayINT); // 
  t16=TWI_INT_timeout;
  

  // wait until SCL is LO //
  while(t16)
  {
       t16--;
       if (!(PIN_USI & (1<<PORT_USI_SCL))) t16=0;    SCL = LOW -> Done
  }
  
  
  */

  switch (USI_TWI_Overflow_State)
  {
    // ---------- Address mode ----------
    // Check address and send ACK (and next USI_SLAVE_SEND_DATA) if OK, else reset USI.
    case USI_SLAVE_CHECK_ADDRESS:
      if ((USIDR == 0) || (( USIDR>>1 ) == TWI_slaveAddress))
      {
        
        #ifdef TWI_DelaySTART_ON
        // this will supress rise again of SCL after Stretch gone 
        _TWIdelay_loop(TWI_DelaySTART); // simple but time consuming
        #endif
        
        
        TWI_Txptr=0;
        TWI_Rxptr=0;
        
        if ( USIDR & 0x01 )
        {
          USI_TWI_Overflow_State = USI_SLAVE_SEND_DATA;
        }
        else
        {
          USI_TWI_Overflow_State = USI_SLAVE_REQUEST_DATA;
        }
        SET_USI_TO_SEND_ACK();
          
      }
      else
      {
        SET_USI_TO_TWI_START_CONDITION_MODE();          // not my Adress.....
        #ifndef USI_DONBLAKE
        USI_TWI_Overflow_State=USI_SLAVE_StateNone;
        #endif 
      }
      break;


    // ----- Master write data mode ------
    // Check reply and goto USI_SLAVE_SEND_DATA if OK, else reset USI.
    case USI_SLAVE_CHECK_REPLY_FROM_SEND_DATA:
      if ( USIDR ) // If NACK, the master does not want more data.
      {
        SET_USI_TO_TWI_START_CONDITION_MODE();
        #ifndef USI_DONBLAKE
        USI_TWI_Overflow_State=USI_SLAVE_StateNone;
        #endif
        return;
      }
    
    // From here we just drop straight into USI_SLAVE_SEND_DATA if the master sent an ACK

    // Copy data from buffer to USIDR and set USI to shift byte. Next USI_SLAVE_REQUEST_REPLY_FROM_SEND_DATA
    case USI_SLAVE_SEND_DATA:

      // Get data from Buffer
      #ifndef TWI_SingleBuffer
      USIDR = TWI_TxBuf[TWI_Txptr];
      #endif
      #ifdef TWI_SingleBuffer
      USIDR = TWI_RxBuf[TWI_Txptr];
      #endif
      if ( TWI_Txptr < TWI_TX_BUFFER_SIZE )        TWI_Txptr++;

      USI_TWI_Overflow_State = USI_SLAVE_REQUEST_REPLY_FROM_SEND_DATA;
      SET_USI_TO_SEND_DATA();
      break;

    // Set USI to sample reply from master. Next USI_SLAVE_CHECK_REPLY_FROM_SEND_DATA
    case USI_SLAVE_REQUEST_REPLY_FROM_SEND_DATA:
      USI_TWI_Overflow_State = USI_SLAVE_CHECK_REPLY_FROM_SEND_DATA;
      SET_USI_TO_READ_ACK();
      break;

    // ----- Master read data mode ------
    // Set USI to sample data from master. Next USI_SLAVE_GET_DATA_AND_SEND_ACK.
    case USI_SLAVE_REQUEST_DATA:
      USI_TWI_Overflow_State = USI_SLAVE_GET_DATA_AND_SEND_ACK;
      SET_USI_TO_READ_DATA();
      break;

    // Copy data from USIDR and send ACK. Next USI_SLAVE_REQUEST_DATA
    case USI_SLAVE_GET_DATA_AND_SEND_ACK:
      // Put data into Buffer
      //tmpUSIDR = USIDR;             // Not necessary, but prevents warnings
      TWI_RxBuf[TWI_Rxptr] = USIDR;
      if (TWI_Rxptr<TWI_RX_BUFFER_SIZE)    TWI_Rxptr++;
      
      USI_TWI_Overflow_State = USI_SLAVE_REQUEST_DATA;
      SET_USI_TO_SEND_ACK();
      break;
  }
}
// internal delay function. 
// 3 cycles per loop run + 7 cycles call&ret
void _TWIdelay_loop(uint8_t count)
{
	__asm__ volatile (
		"1: dec %0" "\n\t"
		"brne 1b"
		: "=r" (count)
		: "0" (count)
	);
}
