#ifndef I2C_H
#define I2C_H

#define TWI_MYAddress   0x7F


// 7+3*n cycles => @16MHz: n=3 for 1s / n=51 fo 10s / 0=48s
//#define  TWI_DelayINT_ON   1
#define  TWI_DelaySTART_ON 1

//#define TWI_DelayINT       7         
#define TWI_DelaySTART     30       /* 6 s */
//#define TWI_INT_timeout   100

// Don Blake changes
//#define USI_DONBLAKE 1

#define TWI_PackLen        4       

#define TWI_PPos_FLAG      0
#define TWI_PPos_TYPE      1
#define TWI_PPos_DATA      2
#define TWI_PPos_CHKSUM    3

#define TWI_CMD_DIR      0x01
#define TWI_CMD_SPEED    0x02
#define TWI_CMD_FREQ     0x03
#define TWI_CMD_MODE     0x04
#define TWI_CMD_VOLT     0x05
#define TWI_CMD_RAMPUP   0x10
#define TWI_CMD_RAMPDWN  0x11
#define TWI_CMD_CONTROL  0x12
#define TWI_CMD_Rangier  0x14
#define TWI_CMD_FASTRead 0x20
#define TWI_CMD_NEWI2CA  0xFF

#define TWI_FLAG_SLAVEREADY  (1<<7)
#define TWI_FLAG_CHECKOK     (1<<2)
#define TWI_FLAG_DATAVALID   (1<<0)

#define TWI_FLAG_DIR         (1<<0)
#define TWI_FLAG_ONOFF       (1<<1)


/************************************************************************************/

volatile unsigned char TWI_RegisterPointer;
//volatile unsigned char  TWI_STATUS_DIR; -> PWM_Flag_CTRL_DIR
//volatile unsigned char  TWI_STATUS_SPEED;
//volatile unsigned char  TWI_STATUS_FREQ; -> LPWM_FMode
//volatile unsigned char  TWI_STATUS_MODE;
//volatile unsigned char  TWI_STATUS_VOLT; -> LPWM_Voltage
//volatile unsigned char  TWI_STATUS_RAMPUP;
//volatile unsigned char  TWI_STATUS_RAMPDWN;
//volatile unsigned char  TWI_STATUS_CONTROL;  -> PWM_Flag_CTRLI2C
//volatile unsigned char  TWI_STATUS_NEWI2CA;

/************************************************************************************/
typedef     unsigned char       uint8_t;
void I2C_INIT(void);
void I2C_TASK(void);
void Flush_TWI_Buffers(void);
void Flush_TWI_RxBuffer(void);
unsigned char USI_TWI_GetnumofDataInReceiveBuffer(void);
void _TWIdelay_loop(uint8_t  count);
//unsigned char TWI_CalcChecksum(unsigned char *buffer,unsigned char ADR, unsigned char LEN);

/************************************************************************************/
/** TWI USI Driver stuff below !                                                    */
/************************************************************************************/


#define TRUE                1
#define FALSE               0

//////////////////////////////////////////////////////////////////
///////////////// Driver Buffer Definitions //////////////////////
//////////////////////////////////////////////////////////////////

// Singlebuffer is same buffer (RX) for RX and TX ! comment out to use 2 buffers
#define TWI_SingleBuffer 1
    
#define TWI_TX_BUFFER_SIZE  (8)
#define TWI_RX_BUFFER_SIZE  (8)

volatile uint8_t TWI_RxBuf[TWI_RX_BUFFER_SIZE];
volatile uint8_t TWI_Rxptr;

#ifndef TWI_SingleBuffer
volatile uint8_t TWI_TxBuf[TWI_TX_BUFFER_SIZE];
#endif
volatile uint8_t TWI_Txptr;


volatile unsigned char TWI_slaveAddress;
//volatile unsigned char USI_TWI_Overflow_State;


#define USI_SLAVE_CHECK_ADDRESS                (0x00)
#define USI_SLAVE_SEND_DATA                    (0x01)
#define USI_SLAVE_REQUEST_REPLY_FROM_SEND_DATA (0x02)
#define USI_SLAVE_CHECK_REPLY_FROM_SEND_DATA   (0x03)
#define USI_SLAVE_REQUEST_DATA                 (0x04)
#define USI_SLAVE_GET_DATA_AND_SEND_ACK        (0x05)
#define USI_SLAVE_StateNone                    (0x11)

//! Device dependent defines

    #define DDR_USI             DDRA
    #define PORT_USI            PORTA
    #define PIN_USI             PINA
    //#define PORT_USI_SDA        PORTA6
    #define PORT_USI_SDA        6
    //#define PORT_USI_SCL        PORTA4
    #define PORT_USI_SCL        4
    //#define PIN_USI_SDA         PINA6
    #define PIN_USI_SDA         6
    //#define PIN_USI_SCL         PINA4
    #define PIN_USI_SCL         4
    #define USI_START_COND_INT  USISIF
    

#define SET_USI_TO_SEND_ACK()                                                                                 \
{                                                                                                             \
        USIDR    =  0;                                              /* Prepare ACK                         */ \
        DDR_USI |=  (1<<PORT_USI_SDA);                              /* Set SDA as output                   */ \
        USISR    =  (0<<USI_START_COND_INT)|(1<<USIOIF)|(1<<USIPF)|(1<<USIDC)|  /* Clear all flags, except Start Cond  */ \
                    (0x0E<<USICNT0);                                /* set USI counter to shift 1 bit. */ \
}

#define SET_USI_TO_READ_ACK()                                                                                 \
{                                                                                                             \
        DDR_USI &=  ~(1<<PORT_USI_SDA);                             /* Set SDA as intput */                   \
        USIDR    =  0;                                              /* Prepare ACK        */                  \
        USISR    =  (0<<USI_START_COND_INT)|(1<<USIOIF)|(1<<USIPF)|(1<<USIDC)|  /* Clear all flags, except Start Cond  */ \
                    (0x0E<<USICNT0);                                /* set USI counter to shift 1 bit. */ \
}


#define SET_USI_TO_TWI_START_CONDITION_MODE()                                                                                     \
{                                                                                                                                 \
  USICR    =  (1<<USISIE)|(0<<USIOIE)|                        /* Enable Start Condition Interrupt. Disable Overflow Interrupt.*/  \
              (1<<USIWM1)|(0<<USIWM0)|                        /* Set USI in Two-wire mode. No USI Counter overflow hold.      */  \
              (1<<USICS1)|(0<<USICS0)|(0<<USICLK)|            /* Shift Register Clock Source = External, positive edge        */  \
              (0<<USITC);                                                                                                         \
  USISR    =  (0<<USI_START_COND_INT)|(1<<USIOIF)|(1<<USIPF)|(1<<USIDC)|  /* Clear all flags, except Start Cond                            */ \
              (0x0<<USICNT0);                                                                                                     \
}




#define SET_USI_TO_SEND_DATA()                                                                               \
{                                                                                                            \
    DDR_USI |=  (1<<PORT_USI_SDA);                                  /* Set SDA as output                  */ \
    USISR    =  (0<<USI_START_COND_INT)|(1<<USIOIF)|(1<<USIPF)|(1<<USIDC)|      /* Clear all flags, except Start Cond */ \
                (0x0<<USICNT0);                                     /* set USI to shift out 8 bits        */ \
}

#define SET_USI_TO_READ_DATA()                                                                               \
{                                                                                                            \
    DDR_USI &= ~(1<<PORT_USI_SDA);                                  /* Set SDA as input                   */ \
    USISR    =  (0<<USI_START_COND_INT)|(1<<USIOIF)|(1<<USIPF)|(1<<USIDC)|      /* Clear all flags, except Start Cond */ \
                (0x0<<USICNT0);                                     /* set USI to shift out 8 bits        */ \
}

#endif
