

#ifndef IO_H
#define IO_H


/*******************************************************************************/
//
// POT   is connected to ADC1/PA1
// DIR   is connected to PA2
// ONOFF is connected to PA3
// OVC   is connected to PA7/PCI7
//
/*******************************************************************************/

/*** OVC ***/
#define CTRL_OVC_PIN     PINA
#define CTRL_OVC_DDR     DDRA
#define CTRL_OVC_PORT    PORTA
#define CTRL_OVC_VAL    (1<<7)

#define CTRL_OVC_INIT    CTRL_OVC_DDR&=~CTRL_OVC_VAL
#define CTRL_OVC_TST     (CTRL_OVC_PIN&CTRL_OVC_VAL)

#define CTRL_OVC_OUT    CTRL_OVC_PORT&=~CTRL_OVC_VAL
#define CTRL_OVC_LEDON  CTRL_OVC_DDR|=CTRL_OVC_VAL
#define CTRL_OVC_LEDOFF CTRL_OVC_DDR&=~CTRL_OVC_VAL
  


/*** Relais ***/
#define CTRL_RELDIR_PORT    PORTA
#define CTRL_RELONOFF_PORT  PORTA
#define CTRL_RELDIR_DDR     DDRA
#define CTRL_RELONOFF_DDR   DDRA
#define CTRL_RELDIR_VAL    (1<<2)
#define CTRL_RELONOFF_VAL  (1<<3)


#define CTRL_RELDIR_INIT      CTRL_RELDIR_DDR|=CTRL_RELDIR_VAL
#define CTRL_RELONOFF_INIT    CTRL_RELONOFF_DDR|=CTRL_RELONOFF_VAL
#define CTRL_RELDIR_ON        CTRL_RELDIR_PORT|=CTRL_RELDIR_VAL
#define CTRL_RELONOFF_ON      CTRL_RELONOFF_PORT|=CTRL_RELONOFF_VAL
#define CTRL_RELDIR_OFF       CTRL_RELDIR_PORT&=~CTRL_RELDIR_VAL
#define CTRL_RELONOFF_OFF     CTRL_RELONOFF_PORT&=~CTRL_RELONOFF_VAL

/*** Switches (Poti Mode) ***/
#define CTRL_SWDIR_PIN     PINA
#define CTRL_SWONOFF_PIN   PINA
#define CTRL_SWDIR_DDR     DDRA
#define CTRL_SWONOFF_DDR   DDRA
#define CTRL_SWDIR_VAL    (1<<2)
#define CTRL_SWONOFF_VAL  (1<<3)


#define CTRL_SWDIR_INIT      CTRL_SWDIR_DDR&=~CTRL_SWDIR_VAL
#define CTRL_SWONOFF_INIT    CTRL_SWONOFF_DDR&=~CTRL_SWONOFF_VAL
#define CTRL_SWDIR_TST       (CTRL_SWDIR_PIN&CTRL_SWDIR_VAL)
#define CTRL_SWONOFF_TST     (CTRL_SWONOFF_PIN&CTRL_SWONOFF_VAL)

#define CTRL_SWDIR_Input     CTRL_SWDIR_INIT;CTRL_RELDIR_OFF;
#define CTRL_SWONOFF_Input   CTRL_SWONOFF_INIT;CTRL_RELONOFF_OFF;


/*************************************************************************/
#define EEP_Adress_I2CADR       0x10



#define EEP_Adress_HardStartup  120
#define EEP_Adress_Steps        140
#define EEP_Adress_END          173

#define EEP_Adress_VoltHardStart    92
#define EEP_Adress_VoltSoftStart    93
#define EEP_Adress_VoltONTime       94
#define EEP_Adress_VoltOFFTime      95
#define EEP_Adress_VoltStep24       96
#define EEP_Adress_VoltStep25       97
#define EEP_Adress_VoltStep26       98
#define EEP_Adress_VoltStep27       99

#define EEP_Adress_CHK              90

/*************************************************************************/

volatile unsigned char CTRL_SW_COUNTER;

void IO_Init(void);
void IO_Switch_Task(void);

void EEP_WriteByte(unsigned char ADRESS,unsigned char DATA);
unsigned char EEP_ReadByte(unsigned char ADRESS);

void EEP_TestChecksum(void);
unsigned int EEP_CalcChecksum(void);

unsigned char EEP_Readbyte(unsigned char Adress);
void EEP_Writebyte(unsigned char Adress, unsigned char DATA);


/*************************************************************************/
/*
  There are the following 16-Bit values
  - Steps Startup Hard:     10
  - Steps Startup Soft:     10
  - Steps 11-27:            17
                            ---
                            37*2=74 Bytes
  There are the following 8-Bit values
  - Startup_Voltage Hard    1
  - Startup_Voltage Soft    1
  - ON-Time Voltage         1
  - OFF-Time Voltage        1
  - Voltage Step 24         1
  - Voltage Step 25         1
  - Voltage Step 26         1
  - Voltage Step 27         1
                            ---
                            8
  
  - Checksum 16Bit          2
                            ----
                            84 Bytes       

*/
#endif

