#include "PWM.h"
#include "IO.h"
#include "ADC.h"
#include <avr\io.h>
#include <avr/interrupt.h>



/******************************************************************/
/* Init PWM                                                       */ 
/******************************************************************/
void PWM_INIT(void)
{

  // Set default values for PWM (=OFF)
  PWM_VPWMVAL=0;
  PWM_LPWMVAL=0;  
  
  // Timer 0 = VPWM in FastPWM Mode (3-> WGM01=WGM00=1, WGM02=0)
  #ifndef MODE_Timer0_NoOVFL
  TCNT0=0xFF-VPWM_PWM_Width;                  // Set resolution
  #endif
  
  OCR0A=PWM_VStep2VPWM(0);                    // Set VPWM      
  LPWM_PIN_LO;                                // CLR OC0A Port
  LPWM_PIN_OUT;                               // OC0A = OUT-VPWM
  TCCR0A|=(1<<WGM01)+(1<<WGM00)+(1<<COM0A1);  // Mode 3 (1), OC0A clr on BOT, set on match
  TCCR0B&=~(1<<WGM02);                        // Mode 3 (2)
  #ifndef MODE_Timer0_NoOVFL
  TIMSK0|=(1<<TOIE0);                         // enable OVFL INT
  #endif
  
  #ifdef MODE_Timer0_NoOVFL
  TCCR0B|=(1<<CS00);                          // MCLK / 8
  #endif
  #ifndef MODE_Timer0_NoOVFL
  TCCR0B|=(1<<CS01);                          // MCLK / 1  = 62,5kHz
  #endif
  
    
  
  // Timer 1 = LPWM in FastPWM (14-> WGM13=WGM12=WGM11=1, WGM10=0, ICR1=is period)
 
  OCR1AH=LPWM_Width_REdge250_H;
  OCR1AL=LPWM_Width_REdge250_L;               // mid for 250 Hz rising edge
  
  OCR1AH=0;
  OCR1AL=0;
  OCR1BH=0;
  OCR1BL=0;
  
  //TIMSK1=(1<<OCIE1B)+(1<<OCIE1A)+(1<<TOIE1);              // enable OCR1A,OCR1B & TOVFL INTs
  TIMSK1=(1<<OCIE1A)+(1<<OCIE1B)+(1<<TOIE1);                // enable OCR1A+B & TOVFL INTs
  TCCR1A=((1<<WGM11));                                      // Mode 14
  TCCR1B=((1<<WGM12)+(1<<WGM13)+(1<<CS11)+(1<<CS10));       // Mode 14, MCLK/64   + Start Timer

  LPWM_Voltage=17;                           // default
  
  
  LPWM_DMode=LPWM_DMode_fixed;                 // 250 Hz OK 
  
  // Settings for test purposes
  //LPWM_DMode=LPWM_DMode_Fvar;           // 250Hz f/8 geht nicht ?
  //LPWM_DMode=LPWM_DMode_Ffixed_VvarS1;         // 250 Hz OK
  //LPWM_DMode=LPWM_DMode_Ffixed_VvarS2;         // 250 Hz OK  
  //LPWM_DMode=LPWM_DMode_FvarVvarS1;           // 250Hz f/8 geht nicht ?
  //LPWM_DMode=LPWM_DMode_FvarVvarS2;           // 250Hz f/8 geht nicht ?
  
  //LPWM_DMode=LPWM_DMode_fixed;                // 80 Hz OK 
  //LPWM_DMode=LPWM_DMode_Fvar;           // 80 Hz OK
  //LPWM_DMode=LPWM_DMode_Ffixed_VvarS1;         // 80 Hz OK
  //LPWM_DMode=LPWM_DMode_Ffixed_VvarS2;         // 80 Hz OK  
  //LPWM_DMode=LPWM_DMode_FvarVvarS1;           // 80 Hz OK
  //LPWM_DMode=LPWM_DMode_FvarVvarS2;           // 80 Hz OK 
  
  //LPWM_VOCR=PWM_VStep2VPWM(LPWM_Voltage);
  //PWM_SetVOCR();
  LPWM_VOCR=PWM_VStep2VPWM(LPWM_Voltage);
  OCR0A=LPWM_VOCR;
  
  LPWM_FMode_Step=1;
  LPWM_FMode=LPWM_FMode_80;                 // default
  //LPWM_FMode=LPWM_FMode_250;                 // default
  //LPWM_FMode=LPWM_FMode_20k;                 // default
  PWM_LPWM_SetFmode();                       // default 
  VPWM_PIN_OUT;                              // enable VPWM
  
}

/******************************************************************/
/*                                                       */ 
/******************************************************************/
void PWM_LPWM_SetFmode(void)
{
   unsigned int ui16;
   
  //if (Step==0)  return;
  
  TCCR0B&=~((1<<CS01)+(1<<CS00));              // CLR + STOP 
  TCCR1B&=~((1<<CS11)+(1<<CS10));              // CLR + STOP

  TCNT1H=0;
  TCNT1L=0;
  

  if (LPWM_FMode==LPWM_FMode_20k)
   {
        ICR1H=0;
        ICR1L=LPWM_Width_20k_L;
      
        LPWM_PIN_LO;                            // CLR OC0B Port
        LPWM_PIN_OUT;                           // OC0B = OUT-VPWM
      
        TCCR1B|=((1<<CS11));                     // MCLK/8   + Start Timer
        TCCR0B|=(1<<CS01);                       // MCLK/8   + Start Timer 
        
        
   }
   else
   {
        if (LPWM_FMode==LPWM_FMode_250) ui16=LPWM_Width_250;
        if (LPWM_FMode==LPWM_FMode_80)  ui16=LPWM_Width_80;
        
        #ifdef MODE_Frise_ON 
        if (LPWM_DMode==LPWM_DMode_Fvar)
        {
            if (LPWM_FMode_Step==2) ui16=ui16 << 1;
            if (LPWM_FMode_Step==4) ui16=ui16 << 2;
            if ((LPWM_FMode_Step==8) && (LPWM_FMode==LPWM_FMode_250)) ui16=ui16 << 3;       
            if ((LPWM_FMode_Step==8) && (LPWM_FMode==LPWM_FMode_80))  ui16=ui16 << 2;       // not 10 Hz !
        }
        ICR1H=(ui16 >> 8) &0xFF;
        ICR1L=(ui16 >> 0) &0xFF;
        #endif
        #ifndef MODE_Frise_ON
        ICR1H=(ui16 >> 8)&0xFF;
        ICR1L=(ui16 >> 0)&0xFF;
        #endif
      LPWM_PIN_LO;                            // CLR OC0B Port
      LPWM_PIN_OUT;                           // OC0B = OUT-VPWM
      //TCCR0A&=~(1<<COM0B1);                 // OC0B removed 
  
      //TIMSK1|=((1<<OCIE1B)+(1<<OCIE1A));      // enable OCR1A,OCR1B 
 
      TCCR1B|=((1<<CS11)+(1<<CS10));          // MCLK/64   + Start Timer
      #ifdef MODE_Timer0_NoOVFL
      TCCR0B|=(1<<CS00);                      // MCLK/1    + Start Timer
      #endif
      #ifndef MODE_Timer0_NoOVFL
      TCCR0B|=(1<<CS01);                      // MCLK/8    + Start Timer
      #endif 
   }


}
/******************************************************************/
/* SET PWM Value (0..100)                                         */ 
/******************************************************************/
void PWM_LPWM_SetPWM(unsigned char PWMValue)
{
  unsigned int ui16;

  
  
  if (PWMValue==0)
   {
      CTRL_RELONOFF_INIT;                   // Output
      CTRL_RELONOFF_OFF;                    // LO = OFF
      PWM_Flagreg&=~PWM_Flag_CTRL_ON;       // CLR Flag
      return;
   }
   
   
   if (PWM_Flagreg & PWM_Flag_LIGHT)   PWM_LPWM_SetFmode(); // restore old FMode 
  
   if (LPWM_FMode==LPWM_FMode_20k)
      {
        
        if (PWMValue < 6) PWMValue=10;
        OCR1BH=0;
        OCR1BL=PWMValue ;
      }
      else
      {        
        if (LPWM_FMode==LPWM_FMode_250) ui16=PWM_PWM20k_2_PWM250(PWMValue);
        if (LPWM_FMode==LPWM_FMode_80)  ui16=PWM_PWM20k_2_PWM80(PWMValue);
        
        #ifdef MODE_Frise_ON 
        if (LPWM_DMode==LPWM_DMode_Fvar)
        {
            if (LPWM_FMode_Step==2) ui16=ui16 << 1;
            if (LPWM_FMode_Step==4) ui16=ui16 << 2;
            
            #ifdef MODE_Frise1to8_ON
            // only for 1-to8 enabled
            if ((LPWM_FMode_Step==8) && (LPWM_FMode==LPWM_FMode_250)) ui16=ui16 << 3;       
            if ((LPWM_FMode_Step==8) && (LPWM_FMode==LPWM_FMode_80))  ui16=ui16 << 2;       // not 10 Hz !
            #endif
        }
        
        if (ui16 < LPWM_Width_min)  ui16=LPWM_Width_min;
        OCR1AH=LPWM_Width_min_H;
        OCR1AL=LPWM_Width_min_L;
        OCR1BH=(ui16 >> 8) &0xFF;
        OCR1BL=(ui16 >> 0) &0xFF;
        #endif
        
        #ifndef MODE_Frise_ON
        if (ui16 < LPWM_Width_min)  ui16=LPWM_Width_min;
        OCR1AH=LPWM_Width_min_H;
        OCR1AL=LPWM_Width_min_L;
        OCR1BH=(ui16 >> 8)&0xFF;
        OCR1BL=(ui16 >> 0)&0xFF;
        #endif
      }
      CTRL_RELONOFF_OFF;                      // set to LO
      CTRL_SWONOFF_Input;                     // set to input
}

/******************************************************************/
/*  INT - Timer0 OVFL                                             */ 
/******************************************************************/
#ifndef MODE_Timer0_NoOVFL
ISR(TIM0_OVF_vect)
{
  TCNT0=0xFF-VPWM_PWM_Width;              // Set resolution      
}
#endif

/******************************************************************/
/*  INT - Timer1 OCB -> OFF PWM 80/250Hz                          */ 
/******************************************************************/

ISR(TIM1_COMPB_vect) __attribute__ ((naked));
ISR(TIM1_COMPB_vect)
{
    LPWM_PIN_LO;                         // CLR OC0B Port
    asm("reti;");
}
/******************************************************************/
/*  INT - Timer1 OCA -> */ 
/******************************************************************/
ISR(TIM1_COMPA_vect)
{
  if (!CTRL_OVC_TST)    // OVC = LO ?
  {
    CTRL_OVC_LEDON;
    LPWM_PIN_LO;
  }
}
/******************************************************************/
/*  INT - Timer1 OVFL -> Start PWM 80/250Hz                       */ 
/******************************************************************/
ISR(TIM1_OVF_vect)
{
    unsigned int SUB;
    unsigned int t16;
    #ifndef DEBUG_VPWM
    OCR0A=LPWM_VOCR;
    #endif
    
    
    if (!PWM_OVCcounter<PWM_OVCcounter_cnt_critical)    
      {
        CTRL_OVC_LEDOFF;
        LPWM_PIN_HI; // SET OC0B Port=HI
      }  
    
    if (LPWM_FMode==LPWM_FMode_20k) 
    {
      if (Timer_20k)  
      {
        Timer_20k--;
        SUB=0;
      }
      else
      {
        Timer_20k=80;
        SUB=1;
      }
    }
    if (LPWM_FMode==LPWM_FMode_250) SUB=1;
    if (LPWM_FMode==LPWM_FMode_80)  SUB=4;  
   
    
    // Timer (SUB) runs with 250Hz

    t16=CTRL_SW_COUNTER ;
    if (t16>=SUB) t16-=SUB;
    if (t16<SUB) t16=0;
    CTRL_SW_COUNTER=t16 ;
    
    t16=Timer_OVCclear ;
    if (t16>=SUB) t16-=SUB;
    if (t16<SUB) t16=0;
    Timer_OVCclear=t16 ;
    
    #ifdef MODE_RampupDwn_ON
    t16=Timer_Rampdwn;
    if (t16>=SUB) t16-=SUB;
    if (t16<SUB) t16=0;
    Timer_Rampdwn=t16;
    t16=Timer_Rampup ;
    if (t16>=SUB) t16-=SUB;
    if (t16<SUB) t16=0;
    Timer_Rampup=t16 ;
    #endif
    
}

/******************************************************************/
/*                                                                */ 
/******************************************************************/
unsigned char PWM_VStep2VPWM(unsigned char Step_lokal)
{
    // convert voltage step to PWM
    unsigned int ui16;
    unsigned char ui8;
    #ifdef MODE_Timer0_NoOVFL
    // -> OUT = 85+((VAL*135)/16)
    if (Step_lokal > VPWM_Step_max) Step_lokal=VPWM_Step_max;
    ui16=Step_lokal;
    ui16=ui16&0xFF;
    ui16=ui16*135;   // * 54
    ui16=ui16 >> 4;  // div by 16
    ui8=ui16 & 0xFF;
    ui8+=85;        // +85
    return(ui8);
    #endif
}
/******************************************************************/
/*                                                                */ 
/******************************************************************/
unsigned int PWM_PWM20k_2_PWM250(unsigned char Value20k)
{
    // convert 20kHz PWM value to 250Hz value (*10)
    unsigned int ui16, ui162, ui168;
    ui16=Value20k;
    ui16=ui16&0xFF;
    ui162=ui16 << 1;
    ui168=ui162 << 2;
    ui16=ui168+ui162;
    return(ui16);
}

/******************************************************************/
/*                                                                */ 
/******************************************************************/
unsigned int PWM_PWM20k_2_PWM80(unsigned char Value20k)
{
    // convert 20kHz PWM value to 80Hz value (*32)
    unsigned int ui16;
    ui16=Value20k;
    ui16=ui16&0xFF;     // LoByte
    ui16=ui16 << 5;    //
    return(ui16);
}

/******************************************************************/
/*                                                                */ 
/******************************************************************/
void PWM_SetNew(unsigned char Step)
{
 unsigned char t8;
     LPWM_Drivestep=Step;
     ADC_Drivestep2Settings(Step,&t8);              // step 2 PWM value
     #ifdef MODE_Frise_ON 
     #ifdef MODE_Frise1to8_ON
     if (Step<=PWM_FRISE_STEP_8) LPWM_FMode_Step=8;
     #endif
     if (Step>PWM_FRISE_STEP_4)  LPWM_FMode_Step=4;
     if (Step>PWM_FRISE_STEP_2)  LPWM_FMode_Step=2;
     if (Step>PWM_FRISE_STEP_1)  LPWM_FMode_Step=1;
     #endif
     asm("cli;");
     PWM_LPWM_SetFmode();
     PWM_LPWM_SetPWM(t8);                         // set new value (LPWM)
     //PWM_SetVOCR();
     LPWM_VOCR=PWM_VStep2VPWM(LPWM_Voltage);
     asm("sei;"); 

}




