#ifndef PWM_H
#define PWM_H


/****************************************************************/
// CPU = TINY44
// MCLK = 16MHz
/****************************************************************/

/*******************************************************************************/
//
// VPWM  is connected to OC0A / PB2 / PIN-5
// LPWM  is connected to OC1B / PA5 / PIN-8
//
/*******************************************************************************/

#define LPWM_PIN_OUT   DDRA|=(1<<5)
#define LPWM_PIN_IN    DDRA&=~(1<<5)
#define LPWM_PIN_HI    PORTA|=(1<<5)
#define LPWM_PIN_LO    PORTA&=~(1<<5)

#define VPWM_PIN_OUT   DDRB|=(1<<2)


/*******************************************************************************/

//#define DEBUG_params 1
//#define DEBUG_POT

/****************/
// set POTI to control VPWM voltage
//#define DEBUG_VPWM 1
#ifdef DEBUG_VPWM
# warning "++++++++++++++++++++++++++ DEBUG: VPWM Test activated ++++++++++++++++++++++++++"
#endif
/****************/
// Rampup and Rampdown is time intervals
//#define MODE_RampupDwn_ON 1

/****************/
// Frequency rise ON/OFF
#define MODE_Frise_ON 1

#ifdef MODE_Frise_ON
# warning "++++++++++++++++++++++++++ MODE: rising Frequency enabled ++++++++++++++++++++++++++"
#endif
/****************/

// Frequency rise ON/OFF
//#define MODE_Frise1to8_ON 1

#ifdef MODE_Frise1to8_ON
# warning "++++++++++++++++++++++++++ MODE: rising Freq. 1:8 enabled ++++++++++++++++++++++++++"
#endif
/****************/



// use Timer0 0x00...0xFF instead of 100
#define MODE_Timer0_NoOVFL 1
#ifdef MODE_Timer0_NoOVFL
# warning "++++++++++++++++++++++++++ MODE: Timer0 NO Overflow activated ++++++++++++++++++++++++++"
#endif

/****************/

// use special constants for "Rangiergang"
#define MODE_Rangiergang
#ifdef MODE_Rangiergang
# warning "++++++++++++++++++++++++++ MODE: Rangiergang                  ++++++++++++++++++++++++++"
#endif

/*******************************************************************************/

/*******************************************************************************/

/**** Set LPWM frequency here ! ****/
#define LPWM_Width_250_L    0xE8
#define LPWM_Width_250_H    0x03
#define LPWM_Width_250      0x03E8

#define LPWM_Width_80_L     0x80
#define LPWM_Width_80_H     0x0C
#define LPWM_Width_80       0x0C80

#define LPWM_Width_20_L     0xC8
#define LPWM_Width_20_H     0x00
#define LPWM_Width_20       0x00C8

#define LPWM_Width_800_L     0xC8
#define LPWM_Width_800_H     0x00
#define LPWM_Width_800       0x00C8

#define LPWM_Width_20k_L    100

#define LPWM_Width_REdge250_H   0x00
#define LPWM_Width_REdge250_L   0x16


#define LPWM_Width_min_L    0x19
#define LPWM_Width_min_H    0x00
#define LPWM_Width_min      0x0019

/**** Set LPWM frequency here END ****/

/**** VPWM ****/
#define VPWM_PWM_Width       100
#define VPWM_PWM_Width3rd    80
#define VPWM_Step_max        20

/**********************************************************************************************/

#define LPWM_FMode_20k  1
#define LPWM_FMode_250  2
#define LPWM_FMode_80   3

#define LPWM_DMode_fixed          0
#define LPWM_DMode_Fvar           1


#define LPWM_DMode_Fvar (1<<0)

#define PWM_Flag_ADCPOT   (1<<0)
#define PWM_Flag_LIGHT    (1<<1)
#define PWM_Flag_CTRLI2C  (1<<2)
#define PWM_Flag_CTRL_DIR (1<<3)
#define PWM_Flag_CTRL_ON  (1<<4)
#define PWM_Flag_Rangier  (1<<5)

#define PWM_OVCtimer_CLR                50
#define PWM_OVCcounter_cnt_critical     5
#define PWM_OVCcounter_time_crit        250




/**********************************************************************************************/
#define PWM_STEP0   0
#define PWM_STEP1   1
#define PWM_STEP2   5
#define PWM_STEP3   7
#define PWM_STEP4   9
#define PWM_STEP5   11
#define PWM_STEP6   13
#define PWM_STEP7   15
#define PWM_STEP8   17
#define PWM_STEP9   19
#define PWM_STEP10  21
#define PWM_STEP11  24
#define PWM_STEP12  27
#define PWM_STEP13  30
#define PWM_STEP14  33
#define PWM_STEP15  36
#define PWM_STEP16  39
#define PWM_STEP17  42
#define PWM_STEP18  45
#define PWM_STEP19  49
#define PWM_STEP20  53
#define PWM_STEP21  57
#define PWM_STEP22  61
#define PWM_STEP23  65
#define PWM_STEP24  70
#define PWM_STEP25  76
#define PWM_STEP26  82
#define PWM_STEP27  88  
#define PWM_STEP28  98


#define PWM_R_STEP0   0
#define PWM_R_STEP1   5
#define PWM_R_STEP2   6
#define PWM_R_STEP3   7
#define PWM_R_STEP4   8
#define PWM_R_STEP5   9
#define PWM_R_STEP6   10
#define PWM_R_STEP7   11
#define PWM_R_STEP8   12
#define PWM_R_STEP9   13
#define PWM_R_STEP10  14
#define PWM_R_STEP11  15
#define PWM_R_STEP12  16
#define PWM_R_STEP13  17
#define PWM_R_STEP14  18
#define PWM_R_STEP15  19
#define PWM_R_STEP16  20
#define PWM_R_STEP17  21
#define PWM_R_STEP18  22
#define PWM_R_STEP19  23
#define PWM_R_STEP20  24
#define PWM_R_STEP21  25
#define PWM_R_STEP22  26
#define PWM_R_STEP23  27
#define PWM_R_STEP24  28
#define PWM_R_STEP25  31
#define PWM_R_STEP26  34
#define PWM_R_STEP27  37  
#define PWM_R_STEP28  40




/**********************************************************************************************/
#ifdef MODE_Frise_ON 
#ifdef MODE_Frise1to8_ON
// also 1:8
#define PWM_FRISE_STEP_8  5
#define PWM_FRISE_STEP_4  10
#define PWM_FRISE_STEP_2  15
#define PWM_FRISE_STEP_1  20
#endif
#ifndef MODE_Frise1to8_ON
// only 1:4
#define PWM_FRISE_STEP_4  8
#define PWM_FRISE_STEP_2  12
#define PWM_FRISE_STEP_1  17
#endif
#endif

/**********************************************************************************************/


#define LPWM_Rampmode_OFF   0
#define LPWM_Rampmode_250m  1
#define LPWM_Rampmode_500m  2
#define LPWM_Rampmode_1s    3

#define LPWM_Ramptime_250m  62
#define LPWM_Ramptime_500m  125
#define LPWM_Ramptime_1s    250

#define LPWM_Ramp_Mode_OFF  0
#define LPWM_Ramp_Mode_UP   1
#define LPWM_Ramp_Mode_DWN  2
/**********************************************************************************************/
/* !!!! DON't foregt to activate DEBUG flag for debugging parameters !!!! */





/* !!!! DON't foregt to activate DEBUG flag for debugging parameters !!!! */
/**********************************************************************************************/

volatile unsigned char  PWM_VPWMVAL;            // DutyCycle for VoltagePWM
volatile unsigned int   PWM_LPWMVAL;            // DutyCycle for LPWM
volatile unsigned int   LPWM_FMode;             // PWM Frequency Range
volatile unsigned int   LPWM_FMode_Step;        // PWM Frequency (Step)
volatile unsigned int   LPWM_DMode;             // PWM Drive Mode
volatile unsigned char  LPWM_Voltage;           // VPWM Voltage (in Steps)
volatile unsigned char  LPWM_VOCR;              // VPWM Voltage (as OCRx)

volatile unsigned char  LPWM_Drivestep;         // LPWM DriveStep for storage

volatile unsigned char  PWM_ADCResult_Vo;         // VOUT result
volatile unsigned int   PWM_ADCResult_POT;        // Value of Pot (if used)

volatile unsigned char  Timer_Statemachine_Control;
volatile unsigned char  PWM_Flagreg; 

volatile unsigned char  PWM_OVCcounter;
volatile unsigned char  Timer_OVCclear;

#ifdef MODE_RampupDwn_ON
volatile unsigned int  Timer_Rampup;
volatile unsigned int  Timer_Rampdwn;
volatile unsigned char Rampup_Mode;
volatile unsigned char Rampdwn_Mode;
volatile unsigned char Ramp_Mode;  
volatile unsigned char Rampup_StepTemp;
volatile unsigned char Rampdwn_StepTemp; 
#endif

volatile unsigned int  Timer_20k;



//#define DEBUG
#ifdef DEBUG
volatile unsigned char PWM_DBG_I[8];
volatile unsigned char PWM_DBG_I_cnt;

volatile unsigned char PWM_DBG_P[8];
volatile unsigned char PWM_DBG_P_cnt;
#endif

/**********************************************************************************************/

void PWM_INIT(void);
void PWM_LPWM_SetPWM(unsigned char PWMValue);
void PWM_LPWM_SetFmode(void);

unsigned char PWM_VStep2VPWM(unsigned char Step);
unsigned int PWM_PWM20k_2_PWM250(unsigned char Value20k);
unsigned int PWM_PWM20k_2_PWM80(unsigned char Value20k);
//void PWM_SetVOCR(void);




/*
------------------------------------------------------------
------------------------------------------------------------
Timer0: 8-Bit, MCLK = 16MHz
direct OutputCompare
Fpwm(0) = ~ 20-25 kHz
-> MCLK/8 = 2MHz
-> 2MHz / 20kHz = 100

direct OutputCompare
-> MCLK/1 = 16 MHz, TOP=0xFF
-> fpwm = 62,5 kHz
-> 256 Steps




------------------------------------------------------------
Timer1: 16-Bit, MCLK = 16MHz
Fpwm(1) = ~ 280Hz / 70Hz, 0..100
Fclk(1) = ~ 28kHz / 7kHz, 0..400
-> MCLK / 8 = 2MHz ; MCLK / 64 = 250kHz ; MCLK / 256 = 62,5kHz ; MCLK / 1024 = 15,6 kHz

indirect Output compare INT:
-> MCLK / 64 = 250kHz
--> 250 Hz  :-> 1000  mul by 10 = (x*8)+(x*2)
--> 80  Hz  :-> 4000  mul by 40 = ((x*8)*4)+(x*8)

#indirect Output compare as timer event:
#25kHz = 40s == 640 clocks
#--> 250 Hz  :-> 25kHz, count to 100
#--> 80  Hz  :-> 25kHz, count to 400


 
------------------------------------------------------------
Voltage PWM:
Voutmax = 15V == 100
VAL 0 = OFF, VAL 1 = 5V, VAL 2 = 5V5
-> 


VAL     VOLT    OUT     OUT-33  VAL*2   (OUT-33)*16  <- /20
1       5       33,33   0,33    2       
20      15      100     67              1072         54

OUT = 33+((VAL*54)/16)
VAL     Voutsoll    PWM  Vout
1       5,0 V       36   5,4V
2       5,5 V       39   5,85V
10      10,0V       66   9,9V
18      14,0V       93   13,95V

------------------------------------------------------------
Voutmax = 15V == 255  -> 58,8mV/Step
VAL 0 = OFF, VAL 1 = 5V, VAL 2 = 5V5, ....

VAL     VOLT    OUT     OUT-85    (OUT-85)*16   /20     ()*32/20  ()*8/20      
1       5       85      0         0             0       0          0
20      15      254     169       2704          135     270       62,



OUT = 85+((VAL*135)/16)
VAL     Voutsoll    PWM  Vout
1       5,0 V       93   5,46V
2       5,5 V       102  6,00V
10      10,0V       169  9,94V
18      14,0V       237  14,0V

------------------------------------------------------------
LPWM D-Mode
// Mode BIT2 BIT1 BIT0
//      S2   S1   Fvar  
// 0    0    0    0  
// 1    0    0    1
// 2    0    1    0
// 4    1    0    0
// 3    0    1    1
// 5    1    0    1


*/
#endif
